% file   : istpgp.pl  (swi version)
% origin author : E Stabler
% origin date: Jun 2004
% purpose: 
% updates: 
%    note! slow! not optimized for speed

:- op(500, xfy, ::). % lexical items
:- op(500, xfy, @). % for selection features

:- compile('../closure'). % Shieber, Schabes & Pereira's 1993 chart-based closure
:- use_module('../tree_display/stepx',[step/5]).
%:- use_module(library(system),[system/1]).
%:- use_module(library(lists),[append/3,member/2,memberchk/2,reverse/2,nth/3]).

% for swi
nth(X,L,A) :- var(X),!,nthmember(L,A,X).
nth(1,[A|_],A) :- !.
nth(N,[_|L],A) :- N>1, N0 is N-1, nth(N0,L,A).

nthmember([A|_],A,1).
nthmember([_|L],A,N) :- nthmember(L,A,N0), N is N0+1.

% file   : istpgp.pl  (sicstus version)
% origin author : E Stabler
% origin date: Jun 2004
% purpose: 
% updates: 
%    note! slow! not optimized for speed

%% top level interface predicates
parse(Input) :- startCategory(Cat), parse(Input,Cat).

parse(Input,StartCat) :-
	parse(Input,StartCat,Derivation),
	portray_clause(derivation:Derivation).

parse(Input,StartCat,Derivation) :-
	write('analyzing...'),ttyflush,
	statistics(runtime,_),
        buildChart(Input),!,
	statistics(runtime,[_,Runtime]),
	nl,portray_clause(runtime:Runtime),
	( bagof(x,NN^II^stored(NN,II),IIL),length(IIL,IIN),portray_clause(chartlength:IIN) ),
	nl,writeAxioms,nl,
	collectDerivation(Input,StartCat,Derivation).
%	sameOrder(Input,Derivation). % to select derivations with leaves in string order, when possible

showParse(Input) :-
	startCategory(StartCat),
	showParse(Input,StartCat).

showParse(Input,StartCat) :-
	parse(Input,StartCat,T),
	portray_clause(derivation:T),
	B=T,X=T,LexItems=[],D=T, % temporarily, just the derivation tree
	step(T,B,X,LexItems,D).

%% the recognizer
buildChart(Input) :-
	setof(Axiom,lexAxiom(Input,Axiom),Axioms),!,
        closure(Axioms). % close Axioms under relations defined by inference/4
buildChart(_).

reducible(Types0) :-
	selectNth(_I,Types0,TypeI,Types1), selectNth(_J,Types1,TypeJ,_Types),
	(   properTypes([TypeI],[]), properAndSaturatedTypes([TypeJ],[])
	;   properTypes([TypeJ],[]), properAndSaturatedTypes([TypeI],[])
	),
	match(TypeI,TypeJ,_NewType,_SemI,_SemJ,_NewSem,_HiddenTerm,_Uncon).

% a tuple is (ListOfCoords:ListOfTypes@Src)
% merge combines a pair of tuples (if they do not overlap, and catsizes=<1)
inference(merge,
          [ (LeftCoords:(LeftTypes:LeftSems)@_),
	    (RightCoords:(RightTypes:RightSems)@_) ],
            (Coords:(Types:Sems)@mrg(N)),
	  [noOverlaps(LeftCoords,RightCoords),
	   \+reducible(LeftTypes), \+reducible(RightTypes), % eager
	   properTypes(LeftTypes,[]), properAndSaturatedTypes(RightTypes,[]),  %proper and saturated
	   nappend(LeftCoords,RightCoords,Coords,N),
	   nappend(LeftTypes,RightTypes,Types,N),
	   nappend(LeftSems,RightSems,Sems,N),
	   reducible(Types)  % eager
	  ]).

% combine two tuples and contract immediately
inference(movecon,
          [ (Coords0:(Types0:Sems0)@_) ],
            ([(X,Z)|Coords]:([NewType|Types]:[NewSem|Sems])@movecon(Y,HiddenTerm,I,J,TypeI,TypeJ)),
	  [selectNth(I,Types0,TypeI,Types1), selectNth(J,Types1,TypeJ,Types),
	   (properTypes([TypeI],[]), properAndSaturatedTypes([TypeJ],[])   %proper and saturated
	   ;properTypes([TypeJ],[]), properAndSaturatedTypes([TypeI],[])
	   ),
	   selectNth(I,Sems0,SemI,Sems1), selectNth(J,Sems1,SemJ,Sems),
	   match(TypeI,TypeJ,NewType,SemI,SemJ,NewSem,HiddenTerm,_Uncon),
	   selectNthPair(Coords0,X,Y,Coords1,I), selectNthPair(Coords1,Y,Z,Coords,J)
	  ]).

match(L0,[r(C)|R],New,Sem0,[SemC|SemR],NewSem,r(C),U) :- !,
	selectLast2(L0,B,L,Sem0,SemB,SemL),
	lte(B,C),
	SemC=SemB,
	append2(L,R,New,SemL,SemR,NewSem),
	append(L0,[r(C)|R],U), !.
match(L0,[C|R],New,Sem0,[SemC|SemR],NewSem,l(C),U) :- atomic(C),
	selectLast2(L0,l(B),L,Sem0,SemB,SemL),
	lte(C,B),
	SemC=SemB,
	append2(L,R,New,SemL,SemR,NewSem),
	append(L0,[C|R],U), !.

lte(A,A).
lte(A,B):- lt(A,C),lte(C,B).

% a tuple is proper iff each tuple is proper and no 2 tuples have same simple type
properTypes([],_).
properTypes([T|Ts],SimpleTypes0) :- properType(T,0,A), onlyLast(A,SimpleTypes0,SimpleTypes), properTypes(Ts,SimpleTypes).

% a type is proper iff it has exactly 1 simple type
properType([],1,_).
properType([r(_)|T],Flag,A) :- !, properType(T,Flag,A).
properType([l(_)|T],Flag,A) :- !, properType(T,Flag,A).
properType([A|T],0,A) :- !, properType(T,1,A).

% when these feature lists are short, not worth overhead of using order
onlyLast(r(_),Ts,Ts) :- !.
onlyLast(l(_),Ts,Ts) :- !.
onlyLast(A,[A|_],_) :- !, fail.
onlyLast(A,[B|Ts0],[B|Ts]) :- !, onlyLast(A,Ts0,Ts).
onlyLast(A,[],[A]).

% a tuple is proper iff each tuple is properAndSaturated and no 2 tuples have same simple type
properAndSaturatedTypes([],_).
properAndSaturatedTypes([[A]|Ts],SimpleTypes0) :-
	atomic(A), onlyLast(A,SimpleTypes0,SimpleTypes), properAndSaturatedTypes(Ts,SimpleTypes).

%% collect derivation

collectDerivation(L,Cat,Tree):-
	length(L,N),
	stored(_,([(0,N)]:([[Cat]]:[[Sem]])@Src)),%!, EPS -- cannot cut here
	write('accepted as category '),write(Cat),write(': '), writeSpacedList(L), nl,
	write('with value: '),writeLFs([[Sem]]), nl,
	retractall(gCntr(_)),
%	buildDerivation(([(0,N)|Coords]:[[Cat]|Cats]@Src),L,Tree). % assumes chart is in stored/2
	buildDerivation(([(0,N)]:([[Cat]]:[[Sem]])@Src),L,Tree). % assumes chart is in stored/2
%	numlex(NumLex), % get numbered list of lex items
%	numlist(D,NumLex,NumList). % represent yield as numbers
collectDerivation(L,Cat,_):-	%EPS
	length(L,N),
	\+ stored(_,([(0,N)]:([[Cat]]:_)@_Src)),
	write('not accepted as category '),write(Cat),write(': '), writeSpacedList(L), nl, fail.

buildDerivation(([(X,Z)|Coords]:([NewType|Types]:[NewSem|Sems])@movecon(Y,HiddenTerm,I,J,TypeI,TypeJ)),Input,
%		([NewType|Types]:Wordses)/[([_MidType|Types]:Wordses)/[L]]) :-  % tpg case with (Mrg,Move)
		([NewType|Types]:Wordses:[NewSem|Sems])/[L]) :-  % tpg case with (Mrg,Move)
	stored(_,([(X,Z)|Coords]:([NewType|Types]:[NewSem|Sems])@movecon(Y,HiddenTerm,I,J,TypeI,TypeJ))),
	selectNthPair(Coords1,Y,Z,Coords,J),
	selectNthPair(Coords0,X,Y,Coords1,I),
	selectNth(J,Types1,TypeJ,Types),
	selectNth(I,Types0,TypeI,Types1), 
	match(TypeI,TypeJ,NewType,_SemI,_SemJ,NewSem,HiddenTerm,_Uncon),
	buildDerivation((Coords0:(Types0:_Sems0)@_),Input,L),
	wordses([(X,Z)|Coords],Input,Wordses).

buildDerivation((Coords:(Types:Sems)@mrg(N)),Input,(Types:Wordses:Sems)/[L,R]) :-
	stored(_,(Coords:(Types:Sems)@mrg(N))),
	nappend(LeftCoords,RightCoords,Coords,N),
	nappend(LeftTypes,RightTypes,Types,N),
	buildDerivation((LeftCoords:(LeftTypes:_LeftSems)@_),Input,L),
	buildDerivation((RightCoords:(RightTypes:_RightSems)@_),Input,R),
	wordses(Coords,Input,Wordses).

buildDerivation((Coords:(Types:Sems)@lex(Words)),_,(Types:Words:Sems)/[]) :-
	stored(_,(Coords:(Types:Sems)@lex(Words))).

%% That's it. The rest is utilities:

wordses([],_,[]).
wordses([(X,Y)|Coords],Input,[Words|Wordses]) :-
	words(X,Y,Input,Words),
	wordses(Coords,Input,Wordses).

words(X,X,_,[]) :- !.
words(0,N,[Word|Ws],[Word|Words]) :- !, N0 is N-1, words(0,N0,Ws,Words).
words(N1,N2,[_|Ws],Words) :- N10 is N1-1, N20 is N2-1, words(N10,N20,Ws,Words).

selectLast([H0,H|T],Last,[H0|Rem]) :- !, selectLast([H|T],Last,Rem).
selectLast([H],H,[]) :- !.

selectLast2([H0,H|T],Last,[H0|Rem],[H02,H2|T2],Last2,[H02|Rem2]) :- !, selectLast2([H|T],Last,Rem,[H2|T2],Last2,Rem2).
selectLast2([H],H,[],[H2],H2,[]) :- !.

append2([],X,X,[],Y,Y).
append2([H|X],Y,[H|Z],[H2|X2],Y2,[H2|Z2]) :- append2(X,Y,Z,X2,Y2,Z2).

noOverlaps(X0,Y0) :- sort(X0,X), sort(Y0,Y), ordLinearizePairs(X,Y), !.

ordLinearizePairs([],_) :- !.
ordLinearizePairs(_,[]) :- !.
ordLinearizePairs([(A,B)|X],Y) :- A==B, !, ordLinearizePairs(X,Y).
ordLinearizePairs(X,[(A,B)|Y]) :- A==B, !, ordLinearizePairs(X,Y).
ordLinearizePairs(X,Y) :- ordLinearizePairs1(X,Y).

ordLinearizePairs1([],_).
ordLinearizePairs1([(A,B)|X],Y0) :- ordInsertPair((A,B),Y0,Y), ordLinearizePairs1(X,Y).

ordInsertPair((A,B),[(C,D)|Y],[(A,B),(C,D)|Y]) :- B=<C, !. % first
ordInsertPair((A,B),[(C,D)],[(C,D),(A,B)]) :- D=<A, !. % last
ordInsertPair((A,B),[(C,D),(E,F)|Y],[(C,D),(A,B),(E,F)|Y]) :- D=<A, B=<E, !. % second
ordInsertPair((A,B),[(C,D),(E,F)|Y0],[(C,D)|Y]) :- A>=F, !, ordInsertPair((A,B),[(E,F)|Y0],Y). % later

lexAxiom(Input,(Coords:(Types:Sems)@lex(Words))) :- (Types::Strings::Sems), wordsCoords(Strings,Input,Coords,Words,[]).

wordsCoords([],_,[],[],_Used).
wordsCoords([[]|Ws],Input,[(N,N)|Coords],[[]|Words],Used) :- !, wordsCoords(Ws,Input,Coords,Words,Used).
wordsCoords([W|Ws],Input,[(N0,N)|Coords],[W|Words],Used) :-
	nth(N,Input,W),
	\+ member(N,Used), % coordinates cannot overlap
	N0 is N-1,
	wordsCoords(Ws,Input,Coords,Words,[N|Used]).

selectNthPair([(X,Y)|Rest],X,Y,Rest,0).
selectNthPair([(X,Y)|Rest0],Z,W,[(X,Y)|Rest],s(N)) :-  selectNthPair(Rest0,Z,W,Rest,N).

selectNth(0,[X|Xs],X,Xs).
selectNth(s(N),[Y|Ys],X,[Y|Xs]) :- selectNth(N,Ys,X,Xs).

sameOrder(L,D) :- yield(D,L,[]).

yield(_/[H|T],L0,L) :- yield(H,L0,L1),yields(T,L1,L).
yield((_Types:Lists)/[],L0,L) :- prefixes(Lists,L0,L).

yields([],L,L).
yields([T|Ts],L0,L) :- yield(T,L0,L1),yields(Ts,L1,L).

prefixes([],L,L).
prefixes([H|T],L0,L) :- selectFirst(H,L0,L1), prefixes(T,L1,L).

selectFirst([],L,L).
selectFirst(W,[W|L],L).

nappend([],L,L,0).
nappend([E|L],M,[E|N],s(Pos)) :- nappend(L,M,N,Pos).

concat(Atom1,Atom2,Atom12) :-
	(atomic(Atom1) -> true ; portray_clause(error: Atom1),abort),
	(atomic(Atom2) -> true ; portray_clause(error: Atom2),abort),
	name(Atom1,L1),name(Atom2,L2),append(L1,L2,L12),name(Atom12,L12).

numlex(L) :- retractall(gCntr(_)), setof(N=(A::B),(A::B,gensym(N)),L).

:- dynamic gCntr/1.
gensym(B) :- var(B),  ( retract(gCntr(C)), B is C+1 ; B=0 ), asserta(gCntr(B)), !.
gensym(_).

writeAxioms :- stored(_,Ax), writeAxiom(Ax), fail.
writeAxioms.

writeAxiom(Ax) :- numbervars(Ax,0,_), print(Ax), nl, fail.
writeAxiom(_).

writeSpacedList(X) :- atomic(X), !, writeCat(X).
writeSpacedList(l(X)) :- !, writeCat(l(X)).
writeSpacedList(r(X)) :- !, writeCat(r(X)).
writeSpacedList([H]) :- !, writeCat(H).
writeSpacedList([H|T]) :- !, writeCat(H), put(32), writeSpacedList(T).

writeType(X) :- atomic(X), !, writeCat(X).
writeType(l(X)) :- !, writeCat(l(X)).
writeType(r(X)) :- !, writeCat(r(X)).
writeType([H]) :- !, writeCat(H).
writeType([H|T]) :- !, writeCat(H), writeType(T).

writeCat(r(X)) :- !, write(X),write('''').
writeCat(l(X)) :- !, write(X),write('`').
writeCat([]) :- !,write([]).
writeCat(H) :- !, write(H).

writeStrings([]) :- !.
writeStrings([H]) :- !, writeSpacedList(H).
writeStrings([H|T]) :- !, writeSpacedList(H), write(','), writeStrings(T).
writeStrings(X) :- writeCat(X).

writeTypes([]) :- !.
writeTypes([H]) :- !, writeType(H).
writeTypes([H|T]) :- !, writeType(H), write(','), writeTypes(T).
writeTypes(X) :- writeCat(X).

writeLFs(A) :- numbervars(A,23,_),writeLFs1(A),fail;true.

writeLFs1([]).
writeLFs1([H,I|T]) :- !,writeLFterm(H),write(','),writeLFs1([I|T]).
writeLFs1([H|T]) :- writeLFterm(H),writeLFs1(T).

writeLFterm([]).
writeLFterm([H,I|T]) :- !,writeLF(H),write('.'),writeLFterm([I|T]).
writeLFterm([H|T]) :- writeLF(H),writeLFterm(T).

writeLF(A:B) :- !, write('('),writeLF(A),write(' '),writeLF(B),write(')').
writeLF(A) :- write(A).

portray((A:(B:C)@D)) :- !,writeTypes(A),write(':'),writeStrings(B),write(':'),writeLF(C),write('@'),write(D).
portray((A:B:C)) :- !,writeTypes(A),write(':'),writeStrings(B),write(':'),writeLFs(C).
portray(A) :- writeTypes(A).
