% file: setup.pl  (swi version)
% origin author : E Stabler
% origin date: Jan 2004
% purpose: load files for CKY-like TPG parser, swi version
% use: This is the top file for loading files and demos.
% updates: 
% todo:

%% operator defs - don't touch these
:- op(500, xfy, ::). % lexical items

%% for tree display
:- ensure_loaded('tree_display/wish_treeSWI').
:- ensure_loaded('tree_display/latex_treeSWI').
:- ensure_loaded('tree_display/pptree').

%% uncomment ONE of the recognizers
%:- ensure_loaded('parser/mpgp').  % simple mPG parser
:- ensure_loaded('parser/mtpgp').  % simple mTPG parser
%:- ensure_loaded('dev/stpgp'). % sTPG parser (Move,GCON combined for speed)
%:- ensure_loaded('dev/istpgp'). % isTPG parser 

%% uncomment ONE grammar
:- ['grammars/g1']. % = g0 + who + empty complementizer
%:- ['grammars/g2']. % copy grammar "aabaab"
%:- ['grammars/g3']. % nonconfig aux2, requires PO parser
%:- ['grammars/g4']. % remnant movement
%:- ['grammars/g5']. % Eng aux
%:- ['dev/ig1']. % = g1 with semantics
%:- ['dev/ig3']. % = g3 with semantics

lexintegrity :- (Fs::Ws::Ss),
	( samelengths(Fs,Ws,Ss)  -> portray_clause(ok:(Fs::Ws::Ss)); portray_clause(illformed:(Fs::Ws::Ss)) ),
	fail ; true.

samelengths([],[],[]).
samelengths([F|Fs],[_W|Ws],[S|Ss]) :- length(F,N), length(S,N), samelengths(Fs,Ws,Ss).

g1(a) :- showParse([a,boy,laughs]).
g1(x) :- showParse([a,boy,and,a,girl,laughs]).
g1(b) :- showParse([a,boy,and,a,girl,praised,a,boy]).
g1(c) :- showParse([a,girl,spoke,to,a,boy]).
g1(d) :- showParse([a,girl,praised,a,boy,who,laughs]).
g1(e) :- showParse([the,girl,who,praised,the,boy,laughs]).
g1(whisland1) :- showParse([the,boy,who,the,girl,who,praised,laughs]).
g1(f) :- showParse([a,girl,who,a,boy,spoke,to,laughs]).
g1(g) :- showParse([a,girl,to,who,a,boy,spoke,laughs]).
g1(whisland2) :- showParse([a,boy,who,a,girl,to,who,spoke,laughs]).
g1(whisland3) :- showParse([a,boy,to,who,a,girl,who,spoke,laughs]).

g2(a) :- showParse([]).
g2(b) :- showParse([a,a]).
g2(c) :- showParse([a,b,a,b]).
g2(d) :- showParse([a,a,b,a,a,b]).
g2(no) :- showParse([a,b,b,a]).

g3(a) :- showParse(['Mary',nom,aux,praises,'John',acc]).
g3(b) :- showParse(['Mary',nom,aux,'John',acc,praises]).
g3(c) :- showParse(['John',acc,aux,praises,'Mary',nom]).
g3(d) :- showParse(['John',acc,aux,'Mary',nom,praises]).
g3(e) :- showParse([praises,aux,'Mary',nom,'John',acc]).
g3(f) :- showParse([praises,aux,'John',acc,'Mary',nom]).
g3(x1) :- showParse([aux,praises,'John',acc,'Mary',nom]).
g3(x2) :- showParse([self,nom,aux,praises,'John',acc]).

g4(a) :- showParse(['Bea',see,'-s','Ben'],'T').

g5(a) :- showParse([the,king,will,'-s',laugh],'C'). %Modal-to-T raising
g5(b) :- showParse([the,king,will,'-s',eat,the,pie],'C').
g5(c) :- showParse([will,'-s',the,king,laugh],'C').  %Modal-to-T,T-to-C raising
g5(d) :- showParse([will,'-s',the,king,eat,the,pie],'C').
g5(e) :- showParse([which,king,will,'-s',laugh],'C'). %Modal-to-T,T-to-C raising,wh-movement
g5(f) :- showParse([which,pie,will,'-s',the,king,eat],'C').
g5(g) :- showParse([the,king,will,'-s',have,laugh,'-en'],'C'). %Have + -en
g5(h) :- showParse([will,'-s',the,king,have,laugh,'-en'],'C'). %T-to-C,Have + -en
g5(i) :- showParse([the,king,have,'-s',laugh,'-en'],'C'). %Have-to-T,v-to-En raising
g5(j) :- showParse([have,'-s',the,king,laugh,'-en'],'C'). %T-to-C,Have-to-T,v-to-En raising
g5(k) :- showParse([the,king,be,'-s',laugh,'-ing'],'C').
g5(l) :- showParse([the,king,will,'-s',be,laugh,'-ing'],'C').
g5(m) :- showParse([the,king,have,'-s',be,'-en',laugh,'-ing'],'C').
g5(n) :- showParse([the,king,will,'-s',have,be,'-en',laugh,'-ing'],'C').
g5(o) :- showParse([have,'-s',he,be,'-en',laugh,'-ing'],'C').
g5(p) :- showParse([will,'-s',the,king,have,be,'-en',laugh,'-ing'],'C').
g5(q) :- showParse([the,king,have,'-s',laugh,'-en'],'C').
g5(r) :- showParse([the,king,laugh,'-s'],'C'). %affix hop
g5(s) :- showParse([which,pie,will,'-s',the,king,have,be,'-en',eat,'-ing'],'C').
g5(x) :- showParse([laugh,'-s',the,king],'C'). % NO

g5(t) :- showParse([who,have,'-s',he,see,'-en'],'C').
g5(u) :- showParse([he,think,'-s',he,have,'-s',laugh,'-en'],'C').
g5(v) :- showParse([what,have,'-s',he,think,'-en',who,he,have,'-s',see,'-en'],'C').

smithma2(a) :- showParse(['''aa~ni','#','''a','-~n',hem,him,'-cud','#','''aapi']).	% `I am moving you'
smithma2(b) :- showParse(['''aapi','#','''a','-~n',hem,him,'-cud','#','''aa~ni']).	% `I am moving you'
smithma1(a) :- showParse(['''aa~ni','#''a~n',hem,himcud,'#','''aapi']).	% `I am moving you'
smithma1(b) :- showParse(['''aapi','#''a~n',hem,himcud,'#','''aa~ni']).	% `I am moving you'
smithma0(a) :- showParse(['''aa~ni','#','''a','-~n',hem,him,'-cud','''aapi']).	% `I am moving you'
smithma0(b) :- showParse(['''aapi','#','''a','-~n',hem,him,'-cud','''aa~ni']).	% `I am moving you'
