/* dot_labtree.pl
 * 
 * write proof tree in dot format out to dottree.dot,
 *   call dot to produce dottree.ps,
 *   then call gv to display dottree.ps
 *
 * E. Stabler, May 1998
 *
*/

:- op( 1050, xfx, =>).	% Right slash
:- op( 300, xfx, /).	% Right slash
:- op( 300, xfx, \).	% Left slash

:- module(dot_tree,[dot_labtree/1]).
:- use_module(library(lists),[append/3,select/3]).
:- use_module(library(system),[system/1]).

dot_labtree(Term) :- 
	tell('dottree.dot'),
	write('digraph G {'),nl,
	write('	node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('	edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('	edge [style=solid,weight=100];'),nl,
	write('	ordering=out;'),nl,
	write('	ranksep=0.3;'),nl,
	write('	nodesep=0.3;'),nl,
	dottree(Term, _Node, -1, ' '),
	write('}'),nl,
	told,
	system('dot -Tps dottree.dot -o dottree.ps'),
	system('gv dottree.ps').

rootlinks([]).
rootlinks([(A->B)|Links]) :-
	moveArc(A,B),
	rootlinks(Links).

dottree(leaf(Term,axiom), Node, Parent, EdgeLabel) :-
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(Term),write('"];'),nl,
	dominanceArc(Parent,Node,EdgeLabel).

dottree(tree(RootLabel,EdgeLabel0,Subtrees), Node, Parent, EdgeLabel) :-
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(RootLabel),write('",height=0.1];'),nl,
	dottrees(Subtrees, Nodes, Node, EdgeLabel0),
	same_rank(Nodes),
	dominanceArc(Parent,Node,EdgeLabel).

same_rank([]).
same_rank([E|L]) :-
	write('	{rank=same; '), scolonlist([E|L]), write('}'), write(';'), nl.

dottrees([], [], _, _).
dottrees([T|Ts], [Node|Nodes], Parent, EdgeLabel) :-
	dottree(T, Node, Parent, EdgeLabel),
	dottrees(Ts, Nodes, Parent, '').  % only label first edge from any given node

dominanceArc(-1,_,_) :- !.
dominanceArc(A,B,'') :-
	write('	'),write(B),write('->'),write(A),write(';'),nl.
dominanceArc(A,B,F) :-
	write('	'),write(B),write('->'),write(A),	
	write(' [fontsize=8,label="'),print(F),write('"]'),
	write(';'),nl.

spacelist([]).
spacelist([E]) :- write(E).
spacelist([E,F|L]) :- write(E), write(' '), spacelist([F|L]).

scolonlist([]).
scolonlist([E|L]) :- write(E), write('; '), scolonlist(L).

:- dynamic gensym_counter/2.
gensym(A) :-
        gensym(n,A).
gensym(A,B) :- var(B), atomic(A),
        (   retract(gensym_counter(A,C)) ;  C=0 ),
        D is C+1,
        asserta(gensym_counter(A,D)),
        concat(A,D,B), !.
concat(A,B,C) :-
        (  atom(A) -> name(A,D), name(B,E), append(D,E,F), name(C,F)
        ;  atom(C) -> name(C,F), name(B,E), append(D,E,F), name(A,D)
        ).


