/* dot_tree.pl
 * 
 * write tree in dot format out to dottree.dot,
 *   and then call dot to produce dottree.ps
 * 
 * E. Stabler, March 1998
 *
 * I switch "shape=triangle" for "peripheries=0"
 * I add "arrowhead=none" 
*/

:- module(dot_tree,[dot_moves/1,dot_tree/1]).

dot_tree(Tree) :- dot_moves([root(Tree)]).

dot_moves(Moves) :-
	select(root(Term),Moves,SubMoves),
	tell('dottree.dot'),
	write('digraph G {'),nl,
	write('	node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('	edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('	edge [style=solid,weight=100];'),nl,
	write('	ordering=out;'),nl,
	write('	ranksep=0.3;'),nl,
	write('	nodesep=0.3;'),nl,
	dottree(Term, _Node, -1),
	dot_submoves(SubMoves),
	write('}'),nl,
	told,
	unix(system('dot -Tps dottree.dot -o dottree.ps')).

dot_moves(Moves) :-
	select(rootlinks(Term,Links),Moves,SubMoves),
	tell('dottree.dot'),
	write('digraph G {'),nl,
	write('	node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('	edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('	edge [style=solid,weight=100];'),nl,
	write('	ordering=out;'),nl,
	write('	ranksep=0.3;'),nl,
	write('	nodesep=0.3;'),nl,
	dottree(Term, _Node, -1),
	rootlinks(Links),
	dot_submoves(SubMoves),
	write('}'),nl,
	told,
	unix(system('dot -Tps dottree.dot -o dottree.ps')).

rootlinks([]).
rootlinks([(A->B)|Links]) :-
	moveArc(A,B),
	rootlinks(Links).

dot_submoves([]).
dot_submoves([hmove(SOURCE,LANDING,F1,F2,Term)|Moves]) :-
	gensym(cluster,SUBGRAPH),
	write('		subgraph '),write(SUBGRAPH),write(' {'),nl,
	write('		node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('		edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('		edge [style=solid,weight=100];'),nl,
	write('		ordering=out;'),nl,
	write('		ranksep=0.3;'),nl,
	write('		nodesep=0.3;'),nl,
	dottree(Term, MovedNode, -1),
	write('	}'),nl,
	hmoveArcOut(SOURCE,MovedNode,F1),
	hmoveArcIn(MovedNode,LANDING,F2),
	dot_submoves(Moves).
dot_submoves([move(SOURCE,LANDING,F1,F2,Term)|Moves]) :-
	gensym(cluster,SUBGRAPH),
	write('		subgraph '),write(SUBGRAPH),write(' {'),nl,
	write('		node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('		edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('		edge [style=solid,weight=100];'),nl,
	write('		ordering=out;'),nl,
	write('		ranksep=0.3;'),nl,
	write('		nodesep=0.3;'),nl,
	dottree(Term, MovedNode, -1),
	write('	}'),nl,
	moveArcOut(SOURCE,MovedNode,F1),
	moveArcIn(MovedNode,LANDING,F2),
	dot_submoves(Moves).
dot_submoves([move(SOURCE,LANDING,Term)|Moves]) :-
	gensym(cluster,SUBGRAPH),
	write('		subgraph '),write(SUBGRAPH),write(' {'),nl,
	write('		node [peripheries=0,fontsize=10,height=0.1];'),nl,
	write('		edge [style=solid,arrowhead=none,weight=100];'),nl,
%	write('		edge [style=solid,weight=100];'),nl,	
	write('		ordering=out;'),nl,
	write('		ranksep=0.3;'),nl,
	write('		nodesep=0.3;'),nl,
	dottree(Term, MovedNode, -1),
	write('	}'),nl,
	moveArcOut(SOURCE,MovedNode),
	moveArcIn(MovedNode,LANDING),
	dot_submoves(Moves).

% empty trees - 3 cases
dottree('$nn'(_Node,Parent)/[], _, Parent) :- !.

dottree('$n'(Node)/[], Node, Parent) :- !,
	write('	'),write(Node),
	write(' [label="",height=0];'),nl,
	dominanceArc(Parent,Node).

dottree('$ll'(Cat,F)/[], Node, Parent) :- !,
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(Cat),write('"];'),nl,
	dominanceArc(Parent,Node,F).

dottree(Cat/[], Node, Parent) :-
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(Cat),write('"];'),nl,
	dominanceArc(Parent,Node).

% nonempty trees - 2 cases
dottree('$n'(Node)/[T|Ts], Node, Parent) :- !,
	write('	'),write(Node),
	write(' [label="",height=0];'),nl,
	dottrees([T|Ts], Nodes, Node),
	same_rank(Nodes),
	dominanceArc(Parent,Node).

dottree('$ll'(Cat,F)/[T|Ts], Node, Parent) :- !,
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(Cat),write('",height=0.1];'),nl,
	dottrees([T|Ts], Nodes, Node),
	same_rank(Nodes),
	dominanceArc(Parent,Node,F).
dottree(Cat/[T|Ts], Node, Parent) :-
	gensym(Node),
	write('	'),write(Node),
	write(' [label="'),print(Cat),write('",height=0.1];'),nl,
	dottrees([T|Ts], Nodes, Node),
	same_rank(Nodes),
	dominanceArc(Parent,Node).

same_rank([]).
same_rank([E|L]) :-
	write('	{rank=same; '), scolonlist([E|L]), write('}'), write(';'), nl.

dottrees([], [], _).
dottrees([T|Ts], Nodes, Parent) :-  T='$nn'(_,_)/_, !, %% SPECIAL CASE
	dottree(T, _Node, Parent),
	dottrees(Ts, Nodes, Parent).
dottrees([T|Ts], [Node|Nodes], Parent) :-
	dottree(T, Node, Parent),
	dottrees(Ts, Nodes, Parent).

moveArc(A,B) :-
	write('	'),write(B),write('->'),write(A),
	write(' [style=dotted,arrowhead=normal]'),write(';'),nl.

moveArcOut(A,B) :-
	write('	'),write(A),write('->'),write(B),
	write(' [arrowhead=normal]'),write(';'),nl.
moveArcIn(A,B) :-
	write('	'),write(A),write('->'),write(B),
	write(' [style=dotted,arrowhead=normal]'),write(';'),nl.


moveArc(A,B,F) :-
	write('	'),write(B),write('->'),write(A),
	write(' [style=dotted,arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.

moveArcOut(A,B,F) :-
	write('	'),write(A),write('->'),write(B),
	write(' [arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.
moveArcIn(A,B,F) :-
	write('	'),write(A),write('->'),write(B),
	write(' [style=dotted,arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.

hmoveArc(A,B,F) :-
	write('	'),write(B),write('->'),write(A),
	write(' [style=dashed,arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.

hmoveArcOut(A,B,F) :-
	write('	'),write(A),write('->'),write(B),
	write(' [arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.
hmoveArcIn(A,B,F) :-
	write('	'),write(B),write('->'),write(A),
	write(' [style=dashed,arrowhead=normal,label="'),print(F),write('"]'),
	write(';'),nl.

dominanceArc(-1,_) :- !.
dominanceArc(A,B) :-
	write('	'),write(A),write('->'),write(B),write(';'),nl.
dominanceArc(A,B,F) :-
	write('	'),write(A),write('->'),write(B),	
	write(' [label="'),print(F),write('"]'),
	write(';'),nl.

spacelist([]).
spacelist([E]) :- write(E).
spacelist([E,F|L]) :- write(E), write(' '), spacelist([F|L]).

scolonlist([]).
scolonlist([E|L]) :- write(E), write('; '), scolonlist(L).

:- dynamic gensym_counter/2.
gensym(A) :-
        gensym(n,A).
gensym(A,B) :-
        var(B),
        atomic(A),
        (   retract(gensym_counter(A,C))
        ;   C=0
        ),
        D is C+1,
        asserta(gensym_counter(A,D)),
        concat(A,D,B),
        !.
concat(A,B,C) :-
        (   atom(A) ->
            name(A,D),
            name(B,E),
            append(D,E,F),
            name(C,F)
        ;   atom(C) ->
            name(C,F),
            name(B,E),
            append(D,E,F),
            name(A,D)
        ).


