/* qtree.pl
 * 
 * format a tree for the latex qtree.sty package
 * 
 * E. Stabler, Aug 2002
 */

:- module(qtree,[qtree/1]).

qtree(Term) :-
	numbervars(Term, 0, _),
	write('\\documentclass{article}'),nl,
	write('\\usepackage{amsmath}'),nl,
	write('\\usepackage{qtree}'),nl,
	write('\\textwidth7in'),nl,
	write('\\textheight8.4in'),nl,
	write('\\oddsidemargin-.25in'),nl,
	write('\\evensidemargin-.25in'),nl,
	write('\\topmargin-.25in'),nl,
	write('\\headheight20pt'),nl,
	write('\\headsep30pt'),nl,
	write('\\footskip20pt'),nl,
	write('\\begin{document}'),nl,
	write('\\Tree '),nl,
	qtree(Term, 0),
	nl,
	write('\\end{document}'),nl,
	fail
	;
	true.

qtree(X/[], Column) :- !,
	tab(Column), 
	write('{'),write(X),write('}').
qtree(Cat/[Child|Ren], Column) :- !,
	tab(Column),write('[ '),
	qtrees([Child|Ren], Column),
	nl,write(' ].'),write(Cat).
qtree(X, Column) :- 
	tab(Column), 
	write('{'),write(X),write('}').

qtrees([], _).
qtrees([Tree|Trees], Column) :-
	NextColumn is Column+4,
	nl, qtree(Tree, NextColumn),
	qrest_trees(Trees, NextColumn).

qrest_trees([], _).
qrest_trees([Tree|Trees], Column) :-
	nl, qtree(Tree, Column),
	qrest_trees(Trees, Column).

%test :- qtree('S'/['NP'/['John'/[]],'IP'/[has/[],'T'/['V'/[seen/[]],'NP'/['the book'/[]]]]]).
%test :- qtree('$\\begin{pmatrix}S\\\\s\\\\\\end{pmatrix}$'/['NP'/['John'/[]],'IP'/[has/[],'T'/['V'/[seen/[]],'NP'/['the book'/[]]]]]).
